@@ -73,6 +73,7 @@ impl Event<'_> {
     /// Generates a byte representation of the value. This should be used when
     /// non-UTF-8 sequences are present or a UTF-8 representation can't be
     /// guaranteed.
+    #[inline]
     #[must_use]
     pub fn to_vec(&self) -> Vec<u8> {
         self.into()
@@ -92,6 +93,7 @@ impl Event<'_> {
     /// not.
     ///
     /// [`clone`]: Self::clone
+    #[inline]
     #[must_use]
     pub fn to_owned(&self) -> Event<'static> {
         match self {
@@ -331,6 +333,7 @@ impl ParsedSectionHeader<'_> {
     /// not.
     ///
     /// [`clone`]: Self::clone
+    #[inline]
     #[must_use]
     pub fn to_owned(&self) -> ParsedSectionHeader<'static> {
         ParsedSectionHeader {
@@ -405,6 +408,7 @@ impl ParsedComment<'_> {
     /// not.
     ///
     /// [`clone`]: Self::clone
+    #[inline]
     #[must_use]
     pub fn to_owned(&self) -> ParsedComment<'static> {
         ParsedComment {
@@ -436,6 +440,7 @@ impl From<ParsedComment<'_>> for Vec<u8> {
 }
 
 impl From<&ParsedComment<'_>> for Vec<u8> {
+    #[inline]
     fn from(c: &ParsedComment) -> Self {
         let mut values = vec![c.comment_tag as u8];
         values.extend(c.comment.iter());
@@ -456,12 +461,14 @@ pub struct Error<'a> {
 impl Error<'_> {
     /// The one-indexed line number where the error occurred. This is determined
     /// by the number of newlines that were successfully parsed.
+    #[inline]
     #[must_use]
     pub const fn line_number(&self) -> usize {
         self.line_number + 1
     }
 
     /// The remaining data that was left unparsed.
+    #[inline]
     #[must_use]
     pub fn remaining_data(&self) -> &[u8] {
         &self.parsed_until
@@ -481,6 +488,7 @@ impl Error<'_> {
     /// not.
     ///
     /// [`clone`]: std::clone::Clone::clone
+    #[inline]
     #[must_use]
     pub fn to_owned(&self) -> Error<'static> {
         Error {
@@ -532,7 +540,17 @@ pub enum ParserOrIoError<'a> {
     Io(std::io::Error),
 }
 
+impl ParserOrIoError<'_> {
+    pub fn into_owned(self) -> ParserOrIoError<'static> {
+        match self {
+            ParserOrIoError::Parser(error) => ParserOrIoError::Parser(error.to_owned()),
+            ParserOrIoError::Io(error) => ParserOrIoError::Io(error),
+        }
+    }
+}
+
 impl Display for ParserOrIoError<'_> {
+    #[inline]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             ParserOrIoError::Parser(e) => e.fmt(f),
@@ -542,6 +560,7 @@ impl Display for ParserOrIoError<'_> {
 }
 
 impl From<std::io::Error> for ParserOrIoError<'_> {
+    #[inline]
     fn from(e: std::io::Error) -> Self {
         Self::Io(e)
     }
@@ -558,6 +577,7 @@ enum ParserNode {
 }
 
 impl Display for ParserNode {
+    #[inline]
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
             Self::SectionHeader => write!(f, "section header"),
@@ -785,6 +805,7 @@ impl<'a> Parser<'a> {
     /// a section) from the parser. Consider [`Parser::take_frontmatter`] if
     /// you need an owned copy only once. If that function was called, then this
     /// will always return an empty slice.
+    #[inline]
     #[must_use]
     pub fn frontmatter(&self) -> &[Event<'a>] {
         &self.frontmatter
@@ -794,6 +815,7 @@ impl<'a> Parser<'a> {
     /// a section) from the parser. Subsequent calls will return an empty vec.
     /// Consider [`Parser::frontmatter`] if you only need a reference to the
     /// frontmatter
+    #[inline]
     pub fn take_frontmatter(&mut self) -> Vec<Event<'a>> {
         let mut to_return = vec![];
         std::mem::swap(&mut self.frontmatter, &mut to_return);
@@ -803,6 +825,7 @@ impl<'a> Parser<'a> {
     /// Returns the parsed sections from the parser. Consider
     /// [`Parser::take_sections`] if you need an owned copy only once. If that
     /// function was called, then this will always return an empty slice.
+    #[inline]
     #[must_use]
     pub fn sections(&self) -> &[ParsedSection<'a>] {
         &self.sections
@@ -811,6 +834,7 @@ impl<'a> Parser<'a> {
     /// Takes the parsed sections from the parser. Subsequent calls will return
     /// an empty vec. Consider [`Parser::sections`] if you only need a reference
     /// to the comments.
+    #[inline]
     pub fn take_sections(&mut self) -> Vec<ParsedSection<'a>> {
         let mut to_return = vec![];
         std::mem::swap(&mut self.sections, &mut to_return);
@@ -818,6 +842,7 @@ impl<'a> Parser<'a> {
     }
 
     /// Consumes the parser to produce a Vec of Events.
+    #[inline]
     #[must_use]
     pub fn into_vec(self) -> Vec<Event<'a>> {
         self.into_iter().collect()
@@ -827,20 +852,18 @@ impl<'a> Parser<'a> {
     #[must_use = "iterators are lazy and do nothing unless consumed"]
     #[allow(clippy::should_implement_trait)]
     pub fn into_iter(self) -> impl Iterator<Item = Event<'a>> + FusedIterator {
-        // Can't impl IntoIter without allocating.and using a generic associated type
-        // TODO: try harder?
-        let section_iter = self.sections.into_iter().flat_map(|section| {
-            vec![Event::SectionHeader(section.section_header)]
-                .into_iter()
-                .chain(section.events)
-        });
-        self.frontmatter.into_iter().chain(section_iter)
+        self.frontmatter.into_iter().chain(
+            self.sections.into_iter().flat_map(|section| {
+                std::iter::once(Event::SectionHeader(section.section_header)).chain(section.events)
+            }),
+        )
     }
 }
 
 impl<'a> TryFrom<&'a str> for Parser<'a> {
     type Error = Error<'a>;
 
+    #[inline]
     fn try_from(value: &'a str) -> Result<Self, Self::Error> {
         parse_from_str(value)
     }
@@ -849,6 +872,7 @@ impl<'a> TryFrom<&'a str> for Parser<'a> {
 impl<'a> TryFrom<&'a [u8]> for Parser<'a> {
     type Error = Error<'a>;
 
+    #[inline]
     fn try_from(value: &'a [u8]) -> Result<Self, Self::Error> {
         parse_from_bytes(value)
     }
@@ -869,7 +893,7 @@ impl<'a> TryFrom<&'a [u8]> for Parser<'a> {
 /// Returns an error if there was an IO error or the read file is not a valid
 /// `git-config` This generally is due to either invalid names or if there's
 /// extraneous data succeeding valid `git-config` data.
-pub fn parse_from_path(path: &Path) -> Result<Parser<'static>, ParserOrIoError> {
+pub fn parse_from_path<P: AsRef<Path>>(path: P) -> Result<Parser<'static>, ParserOrIoError<'static>> {
     let mut bytes = vec![];
     let mut file = std::fs::File::open(path)?;
     file.read_to_end(&mut bytes)?;
@@ -886,6 +910,7 @@ pub fn parse_from_path(path: &Path) -> Result<Parser<'static>, ParserOrIoError>
 /// Returns an error if the string provided is not a valid `git-config`.
 /// This generally is due to either invalid names or if there's extraneous
 /// data succeeding valid `git-config` data.
+#[inline]
 pub fn parse_from_str(input: &str) -> Result<Parser, Error> {
     parse_from_bytes(input.as_bytes())
 }
@@ -1425,12 +1450,12 @@ mod section_headers {
 
     #[test]
     fn newline_in_header() {
-        assert!(section_header(b"[hello\n]").is_err())
+        assert!(section_header(b"[hello\n]").is_err());
     }
 
     #[test]
     fn null_byte_in_header() {
-        assert!(section_header(b"[hello\0]").is_err())
+        assert!(section_header(b"[hello\0]").is_err());
     }
 
     #[test]
@@ -1460,7 +1485,7 @@ mod config_name {
 
     #[test]
     fn cannot_be_empty() {
-        assert!(config_name(b"").is_err())
+        assert!(config_name(b"").is_err());
     }
 }
 
@@ -1711,7 +1736,7 @@ mod section {
                 },
                 3
             ))
-        )
+        );
     }
 
     #[test]
@@ -1824,7 +1849,7 @@ mod section {
                 },
                 0
             ))
-        )
+        );
     }
 
     #[test]
